/*
 * Created on 21.08.2005
 */
package net.hoopajoo.cheeselab3;

import java.io.IOException;
import java.util.*;

//import com.thoughtworks.xstream.XStream;

import jdbm.helper.DefaultSerializer;

/**
 * This is a standalone application for testing the speed of CheeseMap
 * caching infrastructure. Nothing here that an ordinary user would find useful.
 * 
 * @author Antti Andreimann <Antti.Andreimann@mail.ee>
 *
 */
public class SpeedTest {
    static Date rstart = null;

    public static void main(String[] args) {
        final int niter = 10000;
        final int nrooms = 20000;

        // Initialize all needed stuff
        CheeseLab.log_level = 1;
        CheeseMap.props = new Properties();
        CheeseMap.props.setProperty("mapdata_memcache", "false");
        CheeseMap.props.setProperty("mapdata_memcache_preload", "false");
        CheeseMap.cachedir = "testcache";
        CheeseMapData map = new CheeseMapData();

        starttimer("Doing " + niter + " serializations of room data (DefaultSerializer)");
        for( int i = 0; i < niter; i++ ) {
            try {
                byte[] tmp = DefaultSerializer.INSTANCE.serialize( create_room( i + 1 ) );
            } catch (IOException e) {
                CheeseLab.stacktrace();
            }
        }
        endtimer();

        /*XStream xstream = new XStream();
        xstream.alias("room", CheeseRoomData.class );

        starttimer("Doing " + niter + " serialization of room data (XML)");
        for( int i = 0; i < niter; i++ ) {
            String tmp = xstream.toXML( create_room( i + 1 ) );
        }
        endtimer();
        */

        starttimer("Doing " + niter + " encodes/decodes of room data");
        for( int i = 0; i < niter; i++ ) {
            CheeseRoomData tmp = (CheeseRoomData)CheeseDecoder.decode_object(
                                     CheeseDecoder.encode_object( create_room( i + 1 ) ) );
        }
        endtimer();

        starttimer("Injecting " + nrooms + " rooms into cache");
        for( int i = 0; i < nrooms; i++ ) {
            map.roomChanged( create_room(i + 1), null );
        }
        endtimer();

        starttimer("Closing cache (writing to disk)");
        map.closecache();
        endtimer();

        // Reopen cache
        map.initcache();

        starttimer("Doing " + nrooms + " sequential reads from room cache (initial read)");
        for( int i = 0; i < nrooms; i++) {
            CheeseRoomData tmp = map.get_room( i + 1);
        }
        endtimer();

        starttimer("Doing " + niter + " sequential reads from room cache");
        for( int i = 0; i < niter; i++) {
            CheeseRoomData tmp = map.get_room( (i % nrooms) + 1);
        }
        endtimer();

        starttimer("Doing " + niter + " random reads from room cache");
        for( int i = 0; i < niter; i++) {
            CheeseRoomData tmp = map.get_room( (int)(Math.random() * nrooms) + 1 );
        }
        endtimer();

        starttimer("Doing " + niter + " random updates to room cache");
        for( int i = 0; i < niter; i++) {
            map.roomChanged( create_room( (int)(Math.random() * nrooms) ), null );
        }
        endtimer();

        // Reopen cache to discard objects in memory
        map.closecache();
        map.initcache();

        starttimer("Doing " + nrooms + " random reads from room cache (initial read)");
        for( int i = 0; i < nrooms; i++) {
            CheeseRoomData tmp = map.get_room( (int)(Math.random() * nrooms) + 1 );
        }
        endtimer();

        // Reopen cache to discard objects in memory
        map.closecache();
        map.initcache();

        starttimer("Doing " + nrooms + " reads by coordinates (initial read)");
        for( int i = 0; i < nrooms; i++) {
            CheeseRoomData tmp = map.get_room( i+1, i+1, i+1 );
        }
        endtimer();

        starttimer("Doing " + niter + " random reads by coordinates");
        for( int i = 0; i < niter; i++) {
            CheeseRoomData tmp = map.get_room(
                                     (int)(Math.random() * nrooms) + 1,
                                     (int)(Math.random() * nrooms) + 1,
                                     (int)(Math.random() * nrooms) + 1 );
        }
        endtimer();
    }


    public static CheeseRoomData create_room( int id ) {
        CheeseRoomData room = new CheeseRoomData();
        room.roomid = id;
        room.x = id; room.y = id; room.z = id;
        room.name = "CheeseMap speedtest room No. " + id;
        room.edesc = "This is an empty room. There is not a simple thing here. The room itself is\n";
        room.edesc += "completely imaginary. Everywhere You look You see a vast void.\n The";
        room.edesc += "void seems so deep that You feel like it is sucking You straight into it.\n";
        room.edesc += "Every bone and muscle in Your body feels like it's about to disintegrate, to\n";
        room.edesc += "become one with the void, to cease to exist. Your mind feels empty and it's\n";
        room.edesc += "getting difficult to breathe. You desperately want to flee from this place.\n";
        room.edesc += "As You look around for possible exits You see some glyphs magically\n";
        room.edesc += "floating in mid-air. You blink Your eyes in amazement and soon You can read\n";
        room.edesc += "the text as the image slowly fades in:\n";
        room.edesc += "   CheeseMap speedtest room No. " + id + ".\n";
        room.edesc += "   This room will self destruct in 10 seconds!\n";
        room.edesc += "The display in mid-air starts to count down. You gulp and a sudden rush of\n";
        room.edesc += "panic overwhelms You. There seems to be no way out!\n";
        room.terrain = "v";
        room.exit_n = id-1; room.exit_s = id+1;
        room.exit_e = -1; room.exit_w = -1;
        room.exit_u = -1; room.exit_d = -1;
        room.door_s = "void"; room.door_n = "void";
        room.terrain = "v";

        return room;
    }

    public static void starttimer(String out) {
        CheeseLab.dmesg(0, out);
        rstart = new Date();
    }

    public static double endtimer() {
        Date rend = new Date();
        double rdiff = rend.getTime() - rstart.getTime();
        double secs = (double)rdiff / 1000;

        CheeseLab.dmesg(0, "Time: " + rdiff + "ms (" + secs + " secs)" );
        return rdiff;
    }
}
